@@ -10,8 +10,6 @@
 namespace PHPUnit\TextUI\CliArguments;
 
 use const DIRECTORY_SEPARATOR;
-use function array_map;
-use function array_merge;
 use function assert;
 use function basename;
 use function explode;
@@ -19,7 +17,6 @@ use function getcwd;
 use function is_file;
 use function is_numeric;
 use function sprintf;
-use function str_contains;
 use function strtolower;
 use PHPUnit\Event\Facade as EventFacade;
 use PHPUnit\Runner\TestSuiteSorter;
@@ -34,13 +31,17 @@ use SebastianBergmann\CliParser\Parser as CliParser;
  */
 final class Builder
 {
-    private const LONG_OPTIONS = [
+    /**
+     * @var non-empty-list<non-empty-string>
+     */
+    private const array LONG_OPTIONS = [
         'atleast-version=',
         'bootstrap=',
         'cache-result',
         'do-not-cache-result',
         'cache-directory=',
         'check-version',
+        'check-php-configuration',
         'colors==',
         'columns=',
         'configuration=',
@@ -50,6 +51,7 @@ final class Builder
         'coverage-cobertura=',
         'coverage-crap4j=',
         'coverage-html=',
+        'coverage-openclover=',
         'coverage-php=',
         'coverage-text==',
         'only-summary-for-coverage-text',
@@ -57,10 +59,12 @@ final class Builder
         'coverage-xml=',
         'path-coverage',
         'disallow-test-output',
+        'display-all-issues',
         'display-incomplete',
         'display-skipped',
         'display-deprecations',
         'display-phpunit-deprecations',
+        'display-phpunit-notices',
         'display-errors',
         'display-notices',
         'display-warnings',
@@ -88,6 +92,8 @@ final class Builder
         'list-tests',
         'list-tests-xml=',
         'log-junit=',
+        'log-otr=',
+        'include-git-information',
         'log-teamcity=',
         'migrate-configuration',
         'no-configuration',
@@ -99,6 +105,7 @@ final class Builder
         'no-results',
         'order-by=',
         'process-isolation',
+        'do-not-report-useless-tests',
         'dont-report-useless-tests',
         'random-order',
         'random-order-seed=',
@@ -106,14 +113,27 @@ final class Builder
         'reverse-list',
         'static-backup',
         'stderr',
+        'fail-on-all-issues',
         'fail-on-deprecation',
         'fail-on-phpunit-deprecation',
+        'fail-on-phpunit-notice',
+        'fail-on-phpunit-warning',
         'fail-on-empty-test-suite',
         'fail-on-incomplete',
         'fail-on-notice',
         'fail-on-risky',
         'fail-on-skipped',
         'fail-on-warning',
+        'do-not-fail-on-deprecation',
+        'do-not-fail-on-phpunit-deprecation',
+        'do-not-fail-on-phpunit-notice',
+        'do-not-fail-on-phpunit-warning',
+        'do-not-fail-on-empty-test-suite',
+        'do-not-fail-on-incomplete',
+        'do-not-fail-on-notice',
+        'do-not-fail-on-risky',
+        'do-not-fail-on-skipped',
+        'do-not-fail-on-warning',
         'stop-on-defect',
         'stop-on-deprecation==',
         'stop-on-error',
@@ -138,9 +158,11 @@ final class Builder
         'log-events-verbose-text=',
         'version',
         'debug',
+        'with-telemetry',
         'extension=',
     ];
-    private const SHORT_OPTIONS = 'd:c:h';
+
+    private const string SHORT_OPTIONS = 'd:c:h';
 
     /**
      * @var array<string, non-negative-int>
@@ -175,6 +197,7 @@ final class Builder
         $bootstrap                         = null;
         $cacheDirectory                    = null;
         $cacheResult                       = null;
+        $checkPhpConfiguration             = false;
         $checkVersion                      = false;
         $colors                            = null;
         $columns                           = null;
@@ -185,6 +208,7 @@ final class Builder
         $coverageCobertura                 = null;
         $coverageCrap4J                    = null;
         $coverageHtml                      = null;
+        $coverageOpenClover                = null;
         $coveragePhp                       = null;
         $coverageText                      = null;
         $coverageTextShowUncoveredFiles    = null;
@@ -194,10 +218,12 @@ final class Builder
         $defaultTimeLimit                  = null;
         $disableCodeCoverageIgnore         = null;
         $disallowTestOutput                = null;
+        $displayAllIssues                  = null;
         $displayIncomplete                 = null;
         $displaySkipped                    = null;
         $displayDeprecations               = null;
         $displayPhpunitDeprecations        = null;
+        $displayPhpunitNotices             = null;
         $displayErrors                     = null;
         $displayNotices                    = null;
         $displayWarnings                   = null;
@@ -205,14 +231,27 @@ final class Builder
         $excludeGroups                     = null;
         $executionOrder                    = null;
         $executionOrderDefects             = null;
+        $failOnAllIssues                   = null;
         $failOnDeprecation                 = null;
         $failOnPhpunitDeprecation          = null;
+        $failOnPhpunitNotice               = null;
+        $failOnPhpunitWarning              = null;
         $failOnEmptyTestSuite              = null;
         $failOnIncomplete                  = null;
         $failOnNotice                      = null;
         $failOnRisky                       = null;
         $failOnSkipped                     = null;
         $failOnWarning                     = null;
+        $doNotFailOnDeprecation            = null;
+        $doNotFailOnPhpunitDeprecation     = null;
+        $doNotFailOnPhpunitNotice          = null;
+        $doNotFailOnPhpunitWarning         = null;
+        $doNotFailOnEmptyTestSuite         = null;
+        $doNotFailOnIncomplete             = null;
+        $doNotFailOnNotice                 = null;
+        $doNotFailOnRisky                  = null;
+        $doNotFailOnSkipped                = null;
+        $doNotFailOnWarning                = null;
         $stopOnDefect                      = null;
         $stopOnDeprecation                 = null;
         $specificDeprecationToStopOn       = null;
@@ -238,6 +277,8 @@ final class Builder
         $includePath                       = null;
         $iniSettings                       = [];
         $junitLogfile                      = null;
+        $otrLogfile                        = null;
+        $includeGitInformation             = null;
         $listGroups                        = false;
         $listSuites                        = false;
         $listTestFiles                     = false;
@@ -270,6 +311,7 @@ final class Builder
         $printerTestDox                    = null;
         $printerTestDoxSummary             = null;
         $debug                             = false;
+        $withTelemetry                     = false;
         $extensions                        = [];
 
         foreach ($options[0] as $option) {
@@ -277,7 +319,11 @@ final class Builder
 
             switch ($option[0]) {
                 case '--colors':
-                    $colors = $option[1] ?: \PHPUnit\TextUI\Configuration\Configuration::COLOR_AUTO;
+                    $colors = \PHPUnit\TextUI\Configuration\Configuration::COLOR_AUTO;
+
+                    if ($option[1] !== null) {
+                        $colors = $option[1];
+                    }
 
                     break;
 
@@ -346,6 +392,11 @@ final class Builder
 
                     break;
 
+                case '--coverage-openclover':
+                    $coverageOpenClover = $option[1];
+
+                    break;
+
                 case '--coverage-text':
                     if ($option[1] === null) {
                         $option[1] = 'php://stdout';
@@ -454,68 +505,44 @@ final class Builder
                     break;
 
                 case '--group':
-                    if (str_contains($option[1], ',')) {
-                        EventFacade::emitter()->testRunnerTriggeredWarning(
-                            'Using comma-separated values with --group is deprecated and will no longer work in PHPUnit 12. You can use --group multiple times instead.',
-                        );
-                    }
-
                     if ($groups === null) {
                         $groups = [];
                     }
 
-                    $groups = array_merge($groups, explode(',', $option[1]));
+                    $groups[] = $option[1];
 
                     $optionAllowedMultipleTimes = true;
 
                     break;
 
                 case '--exclude-group':
-                    if (str_contains($option[1], ',')) {
-                        EventFacade::emitter()->testRunnerTriggeredWarning(
-                            'Using comma-separated values with --exclude-group is deprecated and will no longer work in PHPUnit 12. You can use --exclude-group multiple times instead.',
-                        );
-                    }
-
                     if ($excludeGroups === null) {
                         $excludeGroups = [];
                     }
 
-                    $excludeGroups = array_merge($excludeGroups, explode(',', $option[1]));
+                    $excludeGroups[] = $option[1];
 
                     $optionAllowedMultipleTimes = true;
 
                     break;
 
                 case '--covers':
-                    if (str_contains($option[1], ',')) {
-                        EventFacade::emitter()->testRunnerTriggeredWarning(
-                            'Using comma-separated values with --covers is deprecated and will no longer work in PHPUnit 12. You can use --covers multiple times instead.',
-                        );
-                    }
-
                     if ($testsCovering === null) {
                         $testsCovering = [];
                     }
 
-                    $testsCovering = array_merge($testsCovering, array_map('strtolower', explode(',', $option[1])));
+                    $testsCovering[] = strtolower($option[1]);
 
                     $optionAllowedMultipleTimes = true;
 
                     break;
 
                 case '--uses':
-                    if (str_contains($option[1], ',')) {
-                        EventFacade::emitter()->testRunnerTriggeredWarning(
-                            'Using comma-separated values with --uses is deprecated and will no longer work in PHPUnit 12. You can use --uses multiple times instead.',
-                        );
-                    }
-
                     if ($testsUsing === null) {
                         $testsUsing = [];
                     }
 
-                    $testsUsing = array_merge($testsUsing, array_map('strtolower', explode(',', $option[1])));
+                    $testsUsing[] = strtolower($option[1]);
 
                     $optionAllowedMultipleTimes = true;
 
@@ -533,17 +560,11 @@ final class Builder
                     break;
 
                 case '--test-suffix':
-                    if (str_contains($option[1], ',')) {
-                        EventFacade::emitter()->testRunnerTriggeredWarning(
-                            'Using comma-separated values with --test-suffix is deprecated and will no longer work in PHPUnit 12. You can use --test-suffix multiple times instead.',
-                        );
-                    }
-
                     if ($testSuffixes === null) {
                         $testSuffixes = [];
                     }
 
-                    $testSuffixes = array_merge($testSuffixes, explode(',', $option[1]));
+                    $testSuffixes[] = $option[1];
 
                     $optionAllowedMultipleTimes = true;
 
@@ -584,6 +605,16 @@ final class Builder
 
                     break;
 
+                case '--log-otr':
+                    $otrLogfile = $option[1];
+
+                    break;
+
+                case '--include-git-information':
+                    $includeGitInformation = true;
+
+                    break;
+
                 case '--log-teamcity':
                     $teamcityLogfile = $option[1];
 
@@ -656,46 +687,231 @@ final class Builder
 
                     break;
 
+                case '--fail-on-all-issues':
+                    $failOnAllIssues = true;
+
+                    break;
+
                 case '--fail-on-deprecation':
+                    $this->warnWhenOptionsConflict(
+                        $doNotFailOnDeprecation,
+                        '--fail-on-deprecation',
+                        '--do-not-fail-on-deprecation',
+                    );
+
                     $failOnDeprecation = true;
 
                     break;
 
                 case '--fail-on-phpunit-deprecation':
+                    $this->warnWhenOptionsConflict(
+                        $doNotFailOnPhpunitDeprecation,
+                        '--fail-on-phpunit-deprecation',
+                        '--do-not-fail-on-phpunit-deprecation',
+                    );
+
                     $failOnPhpunitDeprecation = true;
 
                     break;
 
+                case '--fail-on-phpunit-notice':
+                    $this->warnWhenOptionsConflict(
+                        $doNotFailOnPhpunitNotice,
+                        '--fail-on-phpunit-notice',
+                        '--do-not-fail-on-phpunit-notice',
+                    );
+
+                    $failOnPhpunitNotice = true;
+
+                    break;
+
+                case '--fail-on-phpunit-warning':
+                    $this->warnWhenOptionsConflict(
+                        $doNotFailOnPhpunitWarning,
+                        '--fail-on-phpunit-warning',
+                        '--do-not-fail-on-phpunit-warning',
+                    );
+
+                    $failOnPhpunitWarning = true;
+
+                    break;
+
                 case '--fail-on-empty-test-suite':
+                    $this->warnWhenOptionsConflict(
+                        $doNotFailOnEmptyTestSuite,
+                        '--fail-on-empty-test-suite',
+                        '--do-not-fail-on-empty-test-suite',
+                    );
+
                     $failOnEmptyTestSuite = true;
 
                     break;
 
                 case '--fail-on-incomplete':
+                    $this->warnWhenOptionsConflict(
+                        $doNotFailOnIncomplete,
+                        '--fail-on-incomplete',
+                        '--do-not-fail-on-incomplete',
+                    );
+
                     $failOnIncomplete = true;
 
                     break;
 
                 case '--fail-on-notice':
+                    $this->warnWhenOptionsConflict(
+                        $doNotFailOnNotice,
+                        '--fail-on-notice',
+                        '--do-not-fail-on-notice',
+                    );
+
                     $failOnNotice = true;
 
                     break;
 
                 case '--fail-on-risky':
+                    $this->warnWhenOptionsConflict(
+                        $doNotFailOnRisky,
+                        '--fail-on-risky',
+                        '--do-not-fail-on-risky',
+                    );
+
                     $failOnRisky = true;
 
                     break;
 
                 case '--fail-on-skipped':
+                    $this->warnWhenOptionsConflict(
+                        $doNotFailOnSkipped,
+                        '--fail-on-skipped',
+                        '--do-not-fail-on-skipped',
+                    );
+
                     $failOnSkipped = true;
 
                     break;
 
                 case '--fail-on-warning':
+                    $this->warnWhenOptionsConflict(
+                        $doNotFailOnWarning,
+                        '--fail-on-warning',
+                        '--do-not-fail-on-warning',
+                    );
+
                     $failOnWarning = true;
 
                     break;
 
+                case '--do-not-fail-on-deprecation':
+                    $this->warnWhenOptionsConflict(
+                        $failOnDeprecation,
+                        '--do-not-fail-on-deprecation',
+                        '--fail-on-deprecation',
+                    );
+
+                    $doNotFailOnDeprecation = true;
+
+                    break;
+
+                case '--do-not-fail-on-phpunit-deprecation':
+                    $this->warnWhenOptionsConflict(
+                        $failOnPhpunitDeprecation,
+                        '--do-not-fail-on-phpunit-deprecation',
+                        '--fail-on-phpunit-deprecation',
+                    );
+
+                    $doNotFailOnPhpunitDeprecation = true;
+
+                    break;
+
+                case '--do-not-fail-on-phpunit-notice':
+                    $this->warnWhenOptionsConflict(
+                        $failOnPhpunitNotice,
+                        '--do-not-fail-on-phpunit-notice',
+                        '--fail-on-phpunit-notice',
+                    );
+
+                    $doNotFailOnPhpunitNotice = true;
+
+                    break;
+
+                case '--do-not-fail-on-phpunit-warning':
+                    $this->warnWhenOptionsConflict(
+                        $failOnPhpunitWarning,
+                        '--do-not-fail-on-phpunit-warning',
+                        '--fail-on-phpunit-warning',
+                    );
+
+                    $doNotFailOnPhpunitWarning = true;
+
+                    break;
+
+                case '--do-not-fail-on-empty-test-suite':
+                    $this->warnWhenOptionsConflict(
+                        $failOnEmptyTestSuite,
+                        '--do-not-fail-on-empty-test-suite',
+                        '--fail-on-empty-test-suite',
+                    );
+
+                    $doNotFailOnEmptyTestSuite = true;
+
+                    break;
+
+                case '--do-not-fail-on-incomplete':
+                    $this->warnWhenOptionsConflict(
+                        $failOnIncomplete,
+                        '--do-not-fail-on-incomplete',
+                        '--fail-on-incomplete',
+                    );
+
+                    $doNotFailOnIncomplete = true;
+
+                    break;
+
+                case '--do-not-fail-on-notice':
+                    $this->warnWhenOptionsConflict(
+                        $failOnNotice,
+                        '--do-not-fail-on-notice',
+                        '--fail-on-notice',
+                    );
+
+                    $doNotFailOnNotice = true;
+
+                    break;
+
+                case '--do-not-fail-on-risky':
+                    $this->warnWhenOptionsConflict(
+                        $failOnRisky,
+                        '--do-not-fail-on-risky',
+                        '--fail-on-risky',
+                    );
+
+                    $doNotFailOnRisky = true;
+
+                    break;
+
+                case '--do-not-fail-on-skipped':
+                    $this->warnWhenOptionsConflict(
+                        $failOnSkipped,
+                        '--do-not-fail-on-skipped',
+                        '--fail-on-skipped',
+                    );
+
+                    $doNotFailOnSkipped = true;
+
+                    break;
+
+                case '--do-not-fail-on-warning':
+                    $this->warnWhenOptionsConflict(
+                        $failOnWarning,
+                        '--do-not-fail-on-warning',
+                        '--fail-on-warning',
+                    );
+
+                    $doNotFailOnWarning = true;
+
+                    break;
+
                 case '--stop-on-defect':
                     $stopOnDefect = true;
 
@@ -825,7 +1041,16 @@ final class Builder
 
                     break;
 
+                case '--do-not-report-useless-tests':
+                    $reportUselessTests = false;
+
+                    break;
+
                 case '--dont-report-useless-tests':
+                    EventFacade::emitter()->testRunnerTriggeredPhpunitDeprecation(
+                        'Option --dont-report-useless-tests is deprecated, use --do-not-report-useless-tests instead',
+                    );
+
                     $reportUselessTests = false;
 
                     break;
@@ -850,6 +1075,11 @@ final class Builder
 
                     break;
 
+                case '--display-all-issues':
+                    $displayAllIssues = true;
+
+                    break;
+
                 case '--display-incomplete':
                     $displayIncomplete = true;
 
@@ -870,6 +1100,11 @@ final class Builder
 
                     break;
 
+                case '--display-phpunit-notices':
+                    $displayPhpunitNotices = true;
+
+                    break;
+
                 case '--display-errors':
                     $displayErrors = true;
 
@@ -900,6 +1135,11 @@ final class Builder
 
                     break;
 
+                case '--check-php-configuration':
+                    $checkPhpConfiguration = true;
+
+                    break;
+
                 case '--check-version':
                     $checkVersion = true;
 
@@ -974,6 +1214,11 @@ final class Builder
 
                     break;
 
+                case '--with-telemetry':
+                    $withTelemetry = true;
+
+                    break;
+
                 case '--extension':
                     $extensions[] = $option[1];
 
@@ -987,15 +1232,11 @@ final class Builder
             }
         }
 
-        if (empty($iniSettings)) {
+        if ($iniSettings === []) {
             $iniSettings = null;
         }
 
-        if (empty($coverageFilter)) {
-            $coverageFilter = null;
-        }
-
-        if (empty($extensions)) {
+        if ($extensions === []) {
             $extensions = null;
         }
 
@@ -1008,6 +1249,7 @@ final class Builder
             $bootstrap,
             $cacheDirectory,
             $cacheResult,
+            $checkPhpConfiguration,
             $checkVersion,
             $colors,
             $columns,
@@ -1016,6 +1258,7 @@ final class Builder
             $coverageCobertura,
             $coverageCrap4J,
             $coverageHtml,
+            $coverageOpenClover,
             $coveragePhp,
             $coverageText,
             $coverageTextShowUncoveredFiles,
@@ -1030,14 +1273,27 @@ final class Builder
             $excludeGroups,
             $executionOrder,
             $executionOrderDefects,
+            $failOnAllIssues,
             $failOnDeprecation,
             $failOnPhpunitDeprecation,
+            $failOnPhpunitNotice,
+            $failOnPhpunitWarning,
             $failOnEmptyTestSuite,
             $failOnIncomplete,
             $failOnNotice,
             $failOnRisky,
             $failOnSkipped,
             $failOnWarning,
+            $doNotFailOnDeprecation,
+            $doNotFailOnPhpunitDeprecation,
+            $doNotFailOnPhpunitNotice,
+            $doNotFailOnPhpunitWarning,
+            $doNotFailOnEmptyTestSuite,
+            $doNotFailOnIncomplete,
+            $doNotFailOnNotice,
+            $doNotFailOnRisky,
+            $doNotFailOnSkipped,
+            $doNotFailOnWarning,
             $stopOnDefect,
             $stopOnDeprecation,
             $specificDeprecationToStopOn,
@@ -1063,6 +1319,8 @@ final class Builder
             $includePath,
             $iniSettings,
             $junitLogfile,
+            $otrLogfile,
+            $includeGitInformation,
             $listGroups,
             $listSuites,
             $listTestFiles,
@@ -1088,10 +1346,12 @@ final class Builder
             $testSuite,
             $excludeTestSuite,
             $useDefaultConfiguration,
+            $displayAllIssues,
             $displayIncomplete,
             $displaySkipped,
             $displayDeprecations,
             $displayPhpunitDeprecations,
+            $displayPhpunitNotices,
             $displayErrors,
             $displayNotices,
             $displayWarnings,
@@ -1103,6 +1363,7 @@ final class Builder
             $printerTestDox,
             $printerTestDoxSummary,
             $debug,
+            $withTelemetry,
             $extensions,
         );
     }
@@ -1121,7 +1382,7 @@ final class Builder
         $this->processed[$option]++;
 
         if ($this->processed[$option] === 2) {
-            EventFacade::emitter()->testRunnerTriggeredWarning(
+            EventFacade::emitter()->testRunnerTriggeredPhpunitWarning(
                 sprintf(
                     'Option %s cannot be used more than once',
                     $option,
@@ -1129,4 +1390,22 @@ final class Builder
             );
         }
     }
+
+    /**
+     * @param non-empty-string $option
+     */
+    private function warnWhenOptionsConflict(?bool $current, string $option, string $opposite): void
+    {
+        if ($current === null) {
+            return;
+        }
+
+        EventFacade::emitter()->testRunnerTriggeredPhpunitWarning(
+            sprintf(
+                'Options %s and %s cannot be used together',
+                $option,
+                $opposite,
+            ),
+        );
+    }
 }
